function [alpha,L,Kt,t,Kx,K,hyper_xf,hyper_nf,hyper_tf,hyper_f,hyper_init]...
    = GPR_MTL_MF(X,targetTrainMTL,targetTrainMC,DataCov,TaskCov,taskrank)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Initialisation
no_tasks = max(targetTrainMC);                   % Number of tasks and classes
PoE_task = length(find(targetTrainMC==no_tasks));% Number of points from each task
Ind_task = [];                                   % Task Pointers
FPoE     = [];
noise    = 1e-9;

for i = 1:no_tasks
    Ind_task = [Ind_task i];
    FPoE = [FPoE PoE_task];
end

noT = [FPoE;Ind_task];
t   = noT(2,:);

[dim,N] = size(X);

Sdist   = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Set number of hyperparameters for the data covariance function
if strcmp(DataCov,'covar_ARD_exp');
    dx=dim;Sdist=[];
end
if strcmp(DataCov,'covar_ARD_exp_amp');
    dx=dim+1;Sdist=[];
end
if strcmp(DataCov,'covar_SE_exp');
    dx=2;Sdist=[];
end
if strcmp(DataCov,'covar_NN');
    dx=3;Sdist=[];    
end
if strcmp(DataCov,'covar_Lin');
    dx=1;Sdist=[];
end
if strcmp(DataCov,'covar_Ham');
    dx=1;Sdist=hamdist(X,X);
end
if strcmp(DataCov,'covar_SE_sq');
    dx=2;Sdist=[];
end

if strcmp(TaskCov,'covar_T_cor')||strcmp(TaskCov,'covar_T_cor0');
    no_m=no_tasks^2 -no_tasks;
end
if strcmp(TaskCov,'covar_T_chol');
    no_m=no_tasks^2-(no_tasks^2)/2 +no_tasks/2;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%Initialization of the task hyperparameters (unit-diagonal of Kt)%%%%%%%
if strcmp(TaskCov,'covar_T_cor')
    H=pi/2*ones(no_tasks,no_tasks-1);
    for i=1:no_tasks-1
        H(i,i)=0;
    end
    hyper_t=reshape(H,no_m,1);
end
if strcmp(TaskCov,'covar_T_chol');
    hyper_t=randn(no_m,1);
end
if strcmp(TaskCov,'covar_T_cor0')
    H=2*ones(no_tasks,no_tasks-1);
    for i=1:no_tasks-1
        H(i,i)=-1.3;
    end
    hyper_t=reshape(H,no_m,1);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Randonly generate initial values for the hyperparameters
hyper_x    = randn(dx,1);%Data covariance hyperparameters
hyper_n    = randn(1,1); %noise hyperparameter
hyper_init = [hyper_x;hyper_n;hyper_t];
%%% Options for scg optimisation function
opt_vect      = zeros(1, 18);
opt_vect(2:3) = 1e-4;
opt_vect(4)   = 1e-6;
opt_vect(9)   = 1;
opt_vect(16)  = 1e-8;
opt_vect(17)  = 0.1;

%%% MargL_GPR_MTL  : computes the marginal likelihood 
%%% Grad_GPR_MTL   : computes the gradients with respect to each hyperpar.

hyper_f = scg('MargL_GPR_MTL', hyper_init',opt_vect,'Grad_GPR_MTL',...
    X, targetTrainMTL,noT,targetTrainMC,DataCov,TaskCov,Sdist,taskrank);
hyper_f  = hyper_f';

%%% Separate hyperparameters for the data and task covariance function
hyper_xf = hyper_f(1:dx);
hyper_nf = exp(hyper_f(dx+1));
hyper_tf = hyper_f(dx+2:end);

Kx  = (feval(DataCov,X,X,hyper_xf,Sdist)+noise*eye(N))+hyper_nf*eye(N);
Kt  = feval(TaskCov,t,hyper_tf);       % Compute the task covariance matrix given the optimized task hyperparameters
K   = sparGPMTL(Kt,Kx,t,targetTrainMC);% Compute the final covariance matrix
%%% Compute variables used for prediction
L = chol(K); 
alpha = solve_chol(L,targetTrainMTL);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%