function [PD_E]=Grad_GPR_MTL(hyper,X,Y,noT,t_pointer,DataCov,TaskCov,Sdist,taskrank)
% function that computes  the gradients with
% respect to data and task hyperparameters, to be used  with scg.m.

% Implemetation follows the algorithm for the EP approximation described 
% in Gaussian Processes for Machine learning. 

% cumGauss.m , solve_chol.m : functions taken from Rasmussen's implementation
% for GPs to obtain numerical stability.

% sparGPMTL : function that creates the final covariance matrix(Kt \kron Kx)
% after deleting rows and columns for the unseen outputs.


hyper          = hyper';

NN             = noT(1,:);  % row vector containing the number of data points in each task
[dim,N]        = size(X);   % number of data points in all tasks
t              = noT(2,:);  % task descriptor features to insert into the task cov. function 
noise          = 1e-9;      % Set noise for data covariance function
no_tasks       = length(t); % Number of tasks

% Set number of hyperparameters for the data and task covariance function
% Set functions for gradients
if strcmp(DataCov,'covar_SE_exp');
    DataCov_d = 'covar_SE_exp_dv';
    dx = 2;
end
if strcmp(DataCov,'covar_Lin');
    DataCov_d = 'covar_Lin_dv';
    dx = 1;
end

if strcmp(TaskCov,'covar_T_cor');
    TaskCov_d = 'covar_Td_cor';
    no_m = no_tasks^2 -no_tasks;
end
if strcmp(TaskCov,'covar_T_chol');
    TaskCov_d = 'covar_Td_chol';
    no_m = no_tasks^2-(no_tasks^2)/2 +no_tasks/2;
end
if strcmp(TaskCov,'covar_T_cor0');
    TaskCov_d = 'covar_Td_cor0';
    no_m = no_tasks^2 -no_tasks;
end
if strcmp(TaskCov,'covar_T_Inchol');
    TaskCov_d = 'covar_Td_Inchol';
    no_m = no_tasks*taskrank;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hyper_x  =  hyper(1:dx);
hyper_n  =  exp(hyper(dx+1));
hyper_t  =  hyper(dx+2:end);

Kx          = (feval(DataCov,X,X,hyper_x,Sdist)+noise*eye(N))+hyper_n*eye(N);
[Kt,L_task] = feval(TaskCov,t,hyper_t,taskrank);
KGS         = sparGPMTL(Kt,Kx,t,t_pointer);

L      = chol(KGS); 
% a      = L'\(L\Y);
a=solve_chol(L,Y);
invL   = inv(L);
% invKGS = invL'*inv(L);
invKGS = inv(KGS);



%%%%%%%Gradients for data Covariance hyperparameters
for i=1:length(hyper_x)
    
    kxd = feval(DataCov_d,X,X,hyper_x,Sdist,i);
     
    Cx  = sparGPMTL(Kt,kxd,t,t_pointer);
    
    Gradx(i)=-0.5*(trace((a*a' - invKGS)*Cx));
    
end

%%%%%%%Gradients for noise hyperparameter
kxn   = hyper_n*eye(N);
Cn    = sparGPMTL(Kt,kxn,t,t_pointer);
Gradn = -0.5*(trace((a*a' - invKGS)*Cn));

%%%%%%%Gradients for Task Covariance hyperparameters
for i=1:length(hyper_t)
    [dum,dL_task] = feval(TaskCov_d,t,hyper_t,i,taskrank);
    kfd=dL_task*L_task'+L_task*dL_task';
    Cf=sparGPMTL(kfd,Kx,t,t_pointer);
    Gradf(i)=-0.5*(trace((a*a' - invKGS)*Cf));
end
PD_E=[Gradx';Gradn';Gradf']';