function [logZep]=MargL_GPR_MTL(hyper,X,Y,noT,t_pointer,...
    DataCov,TaskCov,Sdist,taskrank)
% function that computes the log-marginal likelihood given data and task 
% hyperparameters, to be used with scg.m.

% Implemetation follows the algorithm for the EP approximation described 
% in Gaussian Processes for Machine learning. 

% cumGauss.m , solve_chol.m : functions taken from Rasmussen's implementation
% for GPs to obtain numerical stability.
% sparGPMTL : function that creates the final covariance matrix(Kt \kron Kx)
% after deleting rows and columns for the unseen outputs.

hyper          = hyper';

NN             = noT(1,:);  % row vector containing the number of data points in each task
N              = size(X,2); % number of data points in all tasks
t              = noT(2,:);  % task descriptor features to insert into the task cov. function 
dim            = size(X,1);
noise          = 1e-9;      % Set noise for data covariance function
no_tasks       = length(t); % Number of tasks


% Set number of hyperparameters for the data covariance function
% Set functions for gradients
if strcmp(DataCov,'covar_SE_exp');
    dx=2;
end
if strcmp(DataCov,'covar_Lin');
    dx=1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hyper_x  =  hyper(1:dx);
hyper_n  =  exp(hyper(dx+1));
hyper_t  =  hyper(dx+2:end);


Kx          = (feval(DataCov,X,X,hyper_x,Sdist)+noise*eye(N))+hyper_n*eye(N);
[Kt,L_task] = feval(TaskCov,t,hyper_t,taskrank);
KGS         = sparGPMTL(Kt,Kx,t,t_pointer);%Final covariance matrix

L = chol(KGS); 
% a = L'\(L\Y);
a = solve_chol(L,Y);
logZep = -0.5*Y'*a -sum(log(diag(L)))-0.5*N*log(2*pi);

logZep = -logZep;
    
