function [alpha,L,K,hyper_xf,hyper_nf,hyper_f,hyper_init] ...
    = GPR_STL_MF(X,y,DataCov)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Initialisation
noise   = 1e-9;
[dim,N] = size(X);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Set number of hyperparameters for the data covariance function
if strcmp(DataCov,'covar_SE_exp');
    dx=2;Sdist=[];
end
if strcmp(DataCov,'covar_Lin');
    dx=1;Sdist=[];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Randonly generate initial values for the hyperparameters
hyper_x    = randn(dx,1);% Data covariance hyperparameters
hyper_n    = randn(1,1); % noise hyperparameter
hyper_init = [hyper_x;hyper_n];
%%% Options for scg optimisation function
opt_vect      = zeros(1, 18);
opt_vect(2:3) = 1e-4;
opt_vect(4)   = 1e-6;
opt_vect(9)   = 1;
opt_vect(16)  = 1e-8;
opt_vect(17)  = 0.1;
% MargL_GPR_MTL  : computes the marginal likelihood for MTL 
% Grad_GPR_MTL   : computes the gradients with respect to each hyperparameter

hyper_f = scg('MargL_GPR_STL', hyper_init',opt_vect,'Grad_GPR_STL',...
    X, y,DataCov,Sdist);
hyper_f = hyper_f';

% Separate hyperparameters for the data and task covariance function
hyper_xf = hyper_f(1:dx);
hyper_nf = exp(hyper_f(dx+1));

K = (feval(DataCov,X,X,hyper_xf,Sdist)+noise*eye(N))+hyper_nf*eye(N);

%Compute parameters to be used for prediction
L = chol(K); 
alpha = solve_chol(L,y);

