clear 
clc
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Generate data
xmin  = -5;
xmax  = +5;
estr   = xmin:0.01:xmax;
%%% output functions
fyp  = 1+cos(estr)+sin(estr)+0.2*estr;   % function for positively correlated tasks (1,2) 
fyn  = -(1+cos(estr)+sin(estr)+0.2*estr);% function for negatively correlated tasks (3)


no_tasks = 3;  % number of tasks
DPET     = 12; % number of data points per task
gg       = [10 5 4]; 
espace   = floor(length(estr)/DPET);
dnoise   = 0.5;
for j=1:no_tasks
    for i=1:DPET
        if any(j==[1,2])
        xT{j}(i)= estr(i*espace-gg(j));
        yT{j}(i)= fyp(i*espace-gg(j))+dnoise*randn(1);
        else 
        xT{j}(i)= estr(i*espace-gg(j));
        yT{j}(i)= fyn(i*espace-gg(j))+dnoise*randn(1);  
        end   
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Visualise data

figure
plot(xT{1}(1,:),yT{1}(1,:),'k*')
hold on
plot(xT{2}(1,:),yT{2}(1,:),'b+')
plot(xT{3}(1,:),yT{3}(1,:),'r*')
plot(estr,fyp,'k'),hold on,xlabel('x'),ylabel('y')
plot(estr,fyn,'r'),hold on,xlabel('x'),ylabel('y')
legend('Task1','Task2','Task3','fy1','fy2',4)
title('Random samples and functions that generated the data')
fprintf('Press any key to continue \n')
pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Set Covariance functions
DataCov  = 'covar_SE_exp';
TaskCov  = 'covar_T_cor';
taskrank = 2;
%%% Create input matrix and  output vectors
X = [xT{1} xT{2} xT{3}];
targetTrainMTL = [yT{1}';yT{2}';yT{3}'];
targetTrainMC  = [ones(DPET,1);2*ones(DPET,1);3*ones(DPET,1)];

fprintf('Single-task learning hyperparameter optimisation \n')
for j=1:no_tasks
[alpha_s(:,j),L_s(:,:,j),K_s(:,:,j),hyper_xf_s(:,j),hyper_nf_s(:,j),...
    hyper_f_s(:,j),hyper_init_s(:,j)] ...
    = GPR_STL_MF(xT{j},yT{j}',DataCov);
end
fprintf('Multi-task learning hyperparameter optimisation \n')
[alpha_m,L_m,Kt,t,Kx,K_m,hyper_xf_m,hyper_nf_m,...
    hyper_tf_m,hyper_f_m,hyper_init_m]...
    = GPR_MTL_MF(X,targetTrainMTL,targetTrainMC,DataCov,TaskCov,taskrank);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Prediction 
xstar=estr;
for i=1:length(xstar);
    [ mi_mtl, vi_mtl]=test_GPR_MTL(xstar(i),X,alpha_m,L_m,hyper_xf_m,...
        t,targetTrainMC,Kt,DataCov);
    for j=1:no_tasks
        m_mtl(j,i) = mi_mtl(j);
        v_mtl(j,i) = vi_mtl(j);
        [ m_stl(j,i), v_stl(j,i)]=test_GPR_STL...
            (xstar(i),xT{j},alpha_s(:,j),L_s(:,:,j),hyper_xf_s(:,j),DataCov);
    end
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Standard Deviation
std_mtl = sqrt(v_mtl);
std_stl = sqrt(v_stl);
%%% Squared Error between predicted and true function
dummy  = [fyp;fyp;fyn];
ae_mtl = abs(m_mtl-dummy);
ae_stl = abs(m_stl-dummy);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
plot(xstar,m_mtl(1,:),'-k'),hold on
plot(xstar,m_mtl(2,:),'-b'),hold on
plot(xstar,m_mtl(3,:),'-r'),hold on
plot(xstar,m_mtl(1,:)+std_mtl(1,:),'k--')
plot(xstar,m_mtl(1,:)-std_mtl(1,:),'k--')
plot(xT{1}(1,:),yT{1}(1,:),'k+')
plot(xstar,m_mtl(2,:)+std_mtl(2,:),'b--')
plot(xstar,m_mtl(2,:)-std_mtl(2,:),'b--')
plot(xT{2}(1,:),yT{2}(1,:),'b+')
plot(xstar,m_mtl(3,:)+std_mtl(3,:),'r--')
plot(xstar,m_mtl(3,:)-std_mtl(3,:),'r--')
plot(xT{3}(1,:),yT{3}(1,:),'r+')
xlabel('x');ylabel('y')
title('Predictive functions from MTL') 
figure
hintonw(Kt)
xlabel('Task');ylabel('Task')
title('Task Covariance Matrix')
for j=1:no_tasks
    figure,hold on
    plot(xstar,m_mtl(j,:),'-k')
    plot(xstar,m_stl(j,:),'-r')
    if any(j==[1 2])
        plot(estr,fyp,'g')
    else
        plot(estr,fyn,'g')
    end
    legend('MTL','STL','true',4)
    plot(xT{j}(1,:),yT{j}(1,:),'b+')
    plot(xstar,m_mtl(j,:)+std_mtl(j,:),'k--')
    plot(xstar,m_mtl(j,:)-std_mtl(j,:),'k--')
    plot(xstar,m_stl(j,:)+std_stl(j,:),'r--')
    plot(xstar,m_stl(j,:)-std_stl(j,:),'r--')
    xlabel('x'),ylabel('y')
    title(strcat('Comparison of MTL and STL - Task ',num2str(j)))
end
    
    
    
    

for j=1:no_tasks
    figure
    subplot(3,1,1)
    
    plot(xstar,m_mtl(j,:),'-k'),hold on
    if any(j==[1 2])
        plot(estr,fyp,'g')
    else
        plot(estr,fyn,'g')
    end
    legend('MTL','true',2)
    plot(xT{j}(1,:),yT{j}(1,:),'b+')
    plot(xstar,m_mtl(j,:)+std_mtl(j,:),'k--')
    plot(xstar,m_mtl(j,:)-std_mtl(j,:),'k--')
    xlabel('x'),ylabel('y')
    title(strcat('Comparison of MTL and STL - Task ',num2str(j)))
    subplot(3,1,2)
    plot(xstar,m_stl(j,:),'-r'),hold on
    if any(j==[1 2])
        plot(estr,fyp,'g')
    else
        plot(estr,fyn,'g')
    end
    legend('STL','true',2)
    plot(xT{j}(1,:),yT{j}(1,:),'b+')
    plot(xstar,m_stl(j,:)+std_stl(j,:),'r--')
    plot(xstar,m_stl(j,:)-std_stl(j,:),'r--')
    xlabel('x'),ylabel('y')
   
    subplot(3,1,3)
    
    plot(estr,ae_mtl(j,:),'k'),hold on
    plot(estr,ae_stl(j,:),'r')
    legend('MTL','STL',2)
    xlabel('x'),ylabel('Abs. Error')
    title(strcat('Absolute error of MTL and STL - Task ',num2str(j)))

end    
    
    